from .read_powermanga_org import ReadPowerMangaOrg


class DigitalTeam1AltervistaOrg(ReadPowerMangaOrg):
    __title = None
    _name_re = '/reader/[^/]+/([^/]+)'
    _content_str = '{}/reader/read/{}/'
    _chapters_selector = '.chapter_list li > div > a'

    def get_chapters(self):
        self.__title = self.text_content(self.content, 'title')
        return super().get_chapters()

    def __parse_json(self, data) -> list:
        items = []
        for n, i in enumerate(data[0]):
            items.append('{}/reader{}{}{}{}'.format(
                self.domain,
                data[2],  # path
                i['name'],  # image index
                data[1][n],  # image hash
                i['ex']  # image extension
            ))
        return items

    def get_files(self):
        chapter = self.re.search('/(\d+)/', self.chapter).group(1)
        data = {
            'info[manga]': self.manga_name,
            'info[chapter]': chapter,
            'info[ch_sub]': '0',  # todo: watch this
            'info[title]': self.__title,
        }
        json = self.json.loads(self.http_post(
            '{}/reader/c_i'.format(self.domain),
            data=data,
            headers={'X-Requested-With': 'XMLHttpRequest'}
        ))

        if isinstance(json, str):  # DO NOT TOUCH THIS!
            json = self.json.loads(json)

        if json:
            return self.__parse_json(json)
        return []

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img')


main = DigitalTeam1AltervistaOrg



from manga_py.providers.rawdevart_com import RawDevArtCom
from .helpers.std import Std


class MangaTrueCom(RawDevArtCom, Std):
    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/manga/[^/]+/[^\d]*((?:\d+-?)+\d*)')
        idx = re.search(self.chapter).group(1).split('-')
        return '-'.join(idx[:int(len(idx) / 2 + .5)])

    def get_cover(self) -> str:
        return self._cover_from_content('.summary_image img.img-responsive', 'data-src')


main = MangaTrueCom



from manga_py.provider import Provider
from .helpers.std import Std


class FreeAdultComixCom(Provider, Std):

    def get_archive_name(self) -> str:
        return 'archive'

    def get_chapter_index(self) -> str:
        return '0'

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        return [b'']

    def _external_images(self):  # https://freeadultcomix.com/star-vs-the-forces-of-sex-iii-croc/
        links = self._elements('.single-post p > a[target="_blank"] > img')
        items = []
        re = self.re.compile(r'(.+/)th(/.+)')
        for i in links:
            g = re.search(i.get('src')).groups()
            items.append('{}/i/{}/0.jpg'.format(*g))
        return items

    def get_files(self):
        images = self._elements('.single-post p > img[class*="wp-image-"]')
        if not len(images):
            items = self._external_images()
        else:
            items = [i.get('src') for i in images]
        return items

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.get_url()


main = FreeAdultComixCom



from manga_py.provider import Provider
from .helpers.std import Std


class WebToonsCom(Provider, Std):
    __titleNo = 0
    __mainUrl = ''

    def get_archive_name(self) -> str:
        i = self.re.search(r'\.com%s%s' % (
            r'(?:/|%2F)[^/%]+' * 3,
            r'(?:/|%2F)([^/%]+)',
        ), self.chapter)
        return self.normal_arc_name([self.chapter_id, i.group(1)])

    def get_chapter_index(self) -> str:
        return self.re.search(r'\bepisode_no=(\d+)', self.chapter).group(1)

    def get_main_content(self):
        return self.http_get(self.__mainUrl)

    def get_manga_name(self) -> str:
        self.__titleNo = self._get_name(r'title_no=(\d+)')
        name = self._get_name(r'\.com/([^/]+/[^/]+/[^/]+)')
        self.__mainUrl = '{}/{}/list?title_no={}'.format(self.domain, name, self.__titleNo)
        return self._get_name(r'\.com/[^/]+/[^/]+/([^/]+)')

    def _chapters(self, content):
        return self._elements('#_listUl li > a', content)

    def get_chapters(self):
        pages = self._elements('.paginate a + a')
        chapters = self._chapters(self.content)
        if pages:
            n = self.http().normalize_uri
            for i in pages:
                content = self.http_get(n(i.get('href')))
                chapters += self._chapters(content)
        return chapters

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#_imageList img', 'data-url')

    def get_cover(self) -> str:
        img = self.html_fromstring(self.content, '#content > .detail_bg', 0)
        return self.parse_background(img)

    def book_meta(self) -> dict:
        # todo meta
        pass


main = WebToonsCom



from manga_py.provider import Provider
from .helpers.std import Std


class SundayWebryCom(Provider, Std):
    cdn_url = None

    def get_archive_name(self) -> str:
        return self.get_chapter_index()  # Hmm..

    def get_chapter_index(self) -> str:
        return self.re.search(r'cti=([^&]*)', self.chapter).group(1)

    def get_main_content(self):
        if self._storage.get('main_content', None):
            return self._storage.get('main_content')
        return self.http_get('{}/series/{}'.format(
            self.domain,
            self._get_name(r'/series/(\d+)')
        ))

    def get_manga_name(self) -> str:
        return 'sunday_' + self._get_name(r'/series/(\d+)')

    def _chapters(self, content):
        return self._elements('li[id^="product_"] a.button_free', content)

    def get_chapters(self):
        pages = self._elements('.pagination')
        chapters = self._chapters(self.content)
        if pages:
            pages = pages[0].cssselect('a:not([class])')
            n = self.http().normalize_uri
            for i in pages:
                chapters += self._chapters(n(i.get('href')))
        return chapters

    def _prepare_urls(self):
        cid = self.re.search('cid=([^&]+)', self.chapter).group(1)
        license_url = '{}/api4js/contents/license?cid={}'.format(self.domain, cid)
        self.cdn_url = self.json.loads(self.http_get(license_url)).get('url', None)
        items = self.json.loads(self.http_get(self.cdn_url + 'configuration_pack.json'))
        return items.get('configuration', {}).get('contents', [])

    def get_files(self):
        items = []
        for i in self._prepare_urls():
            file = '%s/0.jpeg' % i.get('file')
            items.append(self.cdn_url + file)
        return items

    def after_file_save(self, _path: str, idx: int):  # todo issue #36
        pass

    def get_cover(self) -> str:
        return self._cover_from_content('#series .image > img')

    def prepare_cookies(self):
        self._base_cookies()

    def book_meta(self) -> dict:
        # todo meta
        pass


main = SundayWebryCom



from manga_py.provider import Provider
from .helpers.std import Std


class HentaiFoxCom(Provider, Std):
    __local_storage = None
    _idx_re = r'/g(?:allery)?/(\d+)'
    _url_str = '{}/gallery/{}/'
    _name_selector = '.info h1'

    def get_archive_name(self) -> str:
        return self.get_chapter_index()

    def get_chapter_index(self) -> str:
        return 'archive'

    def get_main_content(self):
        if self.__local_storage is None:
            idx = self._get_name(self._idx_re)
            url = self._url_str.format(self.domain, idx)
            self.__local_storage = self.http_get(url)
        return self.__local_storage

    def get_manga_name(self) -> str:
        return self.text_content(self.content, self._name_selector)

    def get_chapters(self):
        return [b'']

    def get_files(self):
        pages = self._elements('.gallery .preview_thumb a')
        items = []
        n = self.http().normalize_uri
        for i in pages:
            url = self.html_fromstring(n(i.get('href')), '#gimg', 0).get('src')
            items.append(n(url))
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('.cover img')

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.get_url()


main = HentaiFoxCom



from manga_py.provider import Provider
from .helpers.std import Std


class PsychoPlayCo(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        ch = self.chapter
        idx = self.re.search('/read/[^/]+/([^/]+)', ch)
        return idx.group(1)

    def get_main_content(self):
        return self._get_content('{}/series/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/(?:series|read)/([^/]+)')

    def get_chapters(self):
        _chapter = 'a.media-link'
        items = self._elements(_chapter)
        selector = '.pagination li:not([class]) a'
        pages = self.document_fromstring(self.content, selector)
        n = self.http().normalize_uri
        for i in pages:  # TODO! Warning!
            items += self._elements(_chapter, self.http_get(n(i.get('href'))))
        return items

    def get_files(self):  # TODO! Warning!
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.img-responsive', 'data-src')

    def get_cover(self) -> str:
        item = self._elements('.profile-cover-img')
        if item:
            return self.parse_background(item[0])

    def book_meta(self) -> dict:
        # todo meta
        pass


main = PsychoPlayCo



from manga_py.provider import Provider
from .helpers.std import Std


class SubMangaOnline(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('.')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/manga/[^/]+/(\d+(?:\.\d+)?)')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.capitulos-list td > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '#all img.img-responsive', 'data-src')

    def get_cover(self) -> str:
        return self._cover_from_content('.list-group-item .img-responsive')

    def book_meta(self) -> dict:
        return {
            'author': self.text_content(self.content, '.manga .col-sm-8 h5 + h5'),
            'title': self.text_content(self.content, '.manga .col-sm-8 > h2'),
            'annotation': self.text_content(self.content, '.manga .col-sm-8 h5 + .clear20 + div'),
            'keywords': None,
            'cover': self.get_cover(),
            'rating': None,
        }


main = SubMangaOnline



from urllib.parse import quote_plus

import execjs

from manga_py.fs import is_file
from manga_py.provider import Provider
from .helpers.std import Std


class Dm5Com(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'[^\d+](\d+)')
        return re.search(self.chapter[1]).group(1)
        pass

    def get_main_content(self):
        content = self._storage.get('main_content', None)
        if content is None:
            if self.get_url().find('/manhua-'):
                # normal url
                name = self._get_name('/manhua-([^/]+)')
            else:
                # chapter url
                selector = '.title .right-arrow > a'
                name = self.html_fromstring(self.get_url(), selector, 0)
                name = self._get_name('/manhua-([^/]+)', name.get('href'))
            content = self.http_get('{}/manhua-{}/'.format(
                self.domain,
                name
            ))
        return content

    def get_manga_name(self) -> str:
        title = self.text_content(self.content, '.info .title')
        if title:
            return title
        re = self.re.search('/manhua-([^/]+)', self.get_url())
        return re.group(1)

    def get_chapters(self):
        items = self._elements('ul.detail-list-select')
        if not items:
            return []
        items = items[0].cssselect('li > a')
        n = self.http().normalize_uri
        return [(n(i.get('href')), i.text_content()) for i in items]

    def get_files(self):  # fixme
        content = self.http_get(self.chapter[0])
        parser = self.document_fromstring(content)
        pages = parser.cssselect('.chapterpager a')
        if pages:
            pages = int(pages[-1].text_content().strip())
        else:
            pages = 1
        s = lambda k: self.re.search(r'%s\s*=[\s"]*(.+?)[\s"]*;' % k, content).group(1)
        key = parser.cssselect('#dm5_key')[0].get('value')
        cid = s(r'\bDM5_CID')
        mid = s(r'\bDM5_MID')
        sign = s(r'\bDM5_VIEWSIGN')
        sign_dt = quote_plus(s(r'\bDM5_VIEWSIGN_DT'))
        chapter_idx = self.re.search(r'/(m\d+)', self.chapter[0]).group(1)
        url = '{}/{}/chapterfun.ashx?cid={}&page={}&key={}&language=1&gtk=6&_cid={}&_mid={}&_dt={}&_sign={}'
        items = []
        for page in range(pages):
            data = self.http_get(url.format(
                self.domain, chapter_idx,
                cid, page + 1, key, cid,
                mid, sign_dt, sign,
            ), headers=self._get_headers())
            item_url = execjs.eval(data)
            if item_url:
                items += item_url
        return items

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        self._storage['referer'] = self.chapter[0]
        _path, idx, _url = self._save_file_params_helper(url, idx)

        if not is_file(_path):
            self.http(True).download_file(_url, _path, idx)
        callable(callback) and callback()
        self.after_file_save(_path, idx)
        self._archive.lazy_add(_path)
        return _path

    @staticmethod
    def _get_headers():
        return {'Cache-mode': 'no-cache', 'X-Requested-With': 'XMLHttpRequest'}

    def get_cover(self) -> str:
        return self._cover_from_content('.banner_detail_form .cover > img')

    def book_meta(self) -> dict:
        rating = self.text_content(self.content, '.right .score', 0)
        rating = self.re.search(r'(\d\d?\.\d)', rating).group(1)
        author = self.text_content(self.content, '.banner_detail_form .info .subtitle a')
        anno = self.text_content(self.content, '.banner_detail_form .info .content')
        return {
            'author': author,
            'title': self.get_manga_name(),
            'annotation': anno,
            'keywords': str,
            'cover': self.get_cover(),
            'rating': rating,
        }

    def chapter_for_json(self):
        return self.chapter[0]


main = Dm5Com



from manga_py.provider import Provider
from .helpers.std import Std


class MangaShiroNet(Provider, Std):
    alter_re_name = r'\.net/([^/]+)-\d+'
    chapter_re = r'\.net/[^/]+-(\d+(?:-\d+)?)'
    chapters_selector = 'span.leftoff > a'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        return self.re.search(self.chapter_re, chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}/')

    def get_manga_name(self) -> str:
        url = self.get_url()
        if ~url.find('/manga/'):
            re = '/manga/([^/]+)'
        else:
            re = self.alter_re_name
        return self.re.search(re, url).group(1)

    def get_chapters(self):
        return self._elements(self.chapters_selector)

    def get_files(self):
        url = self.chapter
        parser = self.html_fromstring(url)
        items = parser.cssselect('#readerarea a[imageanchor]')
        attr = 'href'
        if not items:
            items = parser.cssselect('#readerarea img[id]')
            attr = 'src'
        return [i.get(attr) for i in items]

    def get_cover(self) -> str:
        return self._cover_from_content('img.attachment-post-thumbnail')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaShiroNet



from manga_py.provider import Provider
from .helpers.std import Std


class TaaddCom(Provider, Std):
    __local_storage = None
    _name_selector = 'h1.chapter_bar a[href*="/book/"]'
    _pages_selector = '#page'
    _chapters_selector = '.chapter_list td[align="left"] a'
    img_selector = '#comicpic'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        return self.remove_not_ascii(self.normal_arc_name([
            self.chapter_id, idx
        ]))

    def get_chapter_index(self) -> str:
        idx = self.re.search('/chapter/([^/]+)/', self.chapter).group(1)
        return idx

    def get_main_content(self):
        return self.http_get('{}/book/{}.html'.format(self.domain, self.manga_name))

    def _re_name(self, url):
        return self.re.search(r'/book/([^/]+)\.html', url)

    def get_manga_name(self) -> str:
        url = self.get_url()
        name = self._re_name(url)
        if not name:
            name = self.html_fromstring(url, self._name_selector, 0).get('href')
            name = self._re_name(name)
        return name.group(1)

    def get_chapters(self):
        return self._elements(self._chapters_selector)

    def prepare_cookies(self):
        self.__local_storage = 0

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        pages = parser.cssselect(self._pages_selector)[0].cssselect('option + option')
        images = self._images_helper(parser, self.img_selector)
        for i in pages:
            c = self.html_fromstring(self.http().normalize_uri(i.get('value')))
            images += self._images_helper(parser, self.img_selector)
        return images

    def book_meta(self) -> dict:
        # todo meta
        pass


main = TaaddCom



from argparse import ArgumentParser

from manga_py.meta import __version__


def _image_args(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Image options')

    args.add_argument('--not-change-files-extension', action='store_const',
                      help='Save files to archive "as is"', const=True, default=False)

    # args.add_argument('--force-png', action='store_const',
    #                          help='Force conversation images to png format', const=True, default=False)
    # args.add_argument('--force-jpg', action='store_const',
    #                          help='Force conversation images to jpg format', const=True, default=False)
    args.add_argument('--no-webp', action='store_const', const=True, default=False,
                             help='Force conversation webp images to jpg format')

    # args.add_argument('-xt', type=int, help='Manual image crop with top side', default=0)
    # args.add_argument('-xr', type=int, help='Manual image crop with right side', default=0)
    # args.add_argument('-xb', type=int, help='Manual image crop with bottom side', default=0)
    # args.add_argument('-xl', type=int, help='Manual image crop with left side', default=0)
    # args.add_argument('--crop-blank', action='store_const', help='Crop white lines on image',
    #                          const=True, default=False)


def _debug_args(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Debug / Simulation options')

    args.add_argument('-h', '--help', action='help', help='show help and exit')
    args.add_argument('--print-json', action='store_const', const=True, default=False,
                      help='Print information about the results in the form of json (after completion)')

    args.add_argument('--simulate', action='store_const', const=True, default=False,
                      help='Do not download the files and do not write anything to disk')

    # args.add_argument('--full-error', action='store_const', const=True, default=False,
    #                   help='Show full stack trace')

    # args.add_argument('-vv', '--log', metavar='info', type='str', help='Verbose log')


def _downloading_args(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Downloading options')

    # args.add_argument('-U', '--update-all', action='store_const',
    #                   help='Update all. Not worked now!', const=True, default=False)
    args.add_argument('-s', '--skip-volumes', metavar='count', type=int,
                      help='Skip volumes', default=0)
    args.add_argument('-c', '--max-volumes', metavar='count', type=int,
                      help='Maximum volumes for downloading 0=All', default=0)
    args.add_argument('--user-agent', type=str, help='Don\'t work from protected sites')
    args.add_argument('--proxy', type=str, help='Http proxy')
    args.add_argument('--reverse-downloading', action='store_const',
                      help='Reverse volumes downloading', const=True, default=False)
    args.add_argument('--rewrite-exists-archives', action='store_const', const=True,
                      default=False)
    args.add_argument('-nm', '--no-multi-threads', action='store_const',
                      help='Disallow multi-threads images downloading', const=True, default=False)
    args.add_argument('--one-thread', action='store_const',
                      help='Disallow multi-threads images downloading', const=True, default=False)
    args.add_argument('--zero-fill', action='store_const', const=True, default=False,
                      help='Adds 0 to the end for all chapters (vol_001.zip -> vol_001-0.zip)')
    args.add_argument('--min-free-space', metavar='Mb', type=int,
                      help='Minimum free disc space', default=100)


def _reader_args(args_parser):  # pragma: no cover
    args = args_parser.add_argument_group('Archive options')

    args.add_argument('--cbz', action='store_const', default=False,
                      const=True, help='Make *.cbz archives (for reader)')

    args.add_argument('--rename-pages', action='store_const', default=False, const=True,
                      help='Normalize images names. (example: 0_page_1.jpg -> 0001.jpg)')


def get_cli_arguments() -> ArgumentParser:  # pragma: no cover
    args_parser = ArgumentParser(add_help=False)
    args = args_parser.add_argument_group('General options')

    args.add_argument('url', metavar='url', type=str, help='Downloaded url')
    args.add_argument('--version', action='version', version=__version__)

    args.add_argument('-n', '--name', metavar='name', type=str, help='Manga name', default='')
    args.add_argument('-d', '--destination', metavar='path', type=str,
                      help='Destination folder (Default = current directory', default='Manga')
    args.add_argument('-np', '--no-progress', metavar='no-progress', action='store_const',
                      const=True, help='Don\'t show progress bar', default=False)
    # future
    # args_parser.add_argument('--server', action='store_const', const=True, help='Run web interface',
    #                          default=False)

    _image_args(args_parser)
    _reader_args(args_parser)
    _downloading_args(args_parser)
    _debug_args(args_parser)

    return args_parser



from manga_py.provider import Provider
from .helpers.std import Std


class MangaFoxMe(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        """
        examples:
        <domain>/manga/<name>/v03/c020/1.html
        <domain>/manga/<name>/v03/c020.5/1.html
        <domain>/manga/<name>/v01/c019/1.html
        <domain>/manga/<name>/c022/1.html
        <domain>/manga/<name>/c021/1.html
        <domain>/manga/<name>/v04/c016.5/1.html
        <domain>/manga/<name>/v18/c154.5/1.html
        <domain>/manga/<name>/vTBD/c169/1.html
        <domain>/manga/<name>/v06/c018.5/1.html]
        <domain>/manga/<name>/v03/c020.5/1.html
        <domain>/manga/<name>/v08/c031/1.html
        """
        selector = r'/manga/[^/]+/(?:(v[^/]+)/)?c([^/]+)/'
        groups = self.re.search(selector, self.chapter).groups()
        idx = groups[1].replace('.', '-')
        if not ~idx.find('-'):
            idx = idx + '-0'
        if groups[0]:
            return '{}-{}'.format(idx, groups[0])
        return idx

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)/?')

    def get_chapters(self):
        return self._elements('#chapters a.tips')

    def __get_files_url(self):
        volume = self.chapter
        url = self.http().normalize_uri(volume)
        if ~url.find('.html'):
            url = url[: url.rfind('/')]
        return url

    def get_files(self):
        img_selector = 'img#image'
        _url = self.__get_files_url()
        url = '{}/1.html'.format(_url)
        selector = '#top_bar .r .l select.m'
        parser = self.html_fromstring(url)
        images = self._images_helper(parser, img_selector)
        for page in self._first_select_options(parser, selector, True):
            n = page.get('value')
            if int(n) < 2:  # first / comments page
                continue
            url = '{}/{}.html'.format(_url, n)
            parser = self.html_fromstring(url)
            images += self._images_helper(parser, img_selector)
        return images

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaFoxMe



from .comico_co_id_titles import ComicoCoIdTitles
from .helpers.std import Std


class ComicoCoIdContent(ComicoCoIdTitles, Std):  # maybe
    __origin_url = None

    def get_archive_name(self) -> str:
        return '0'

    def get_chapter_index(self) -> str:
        re = self.re.compile(r'/title/(\d+)')
        return re.search(self.chapter).group(1)

    def get_main_content(self):
        idx = self.re.search(r'contentId=(\d+)', self.get_url())
        return self.http_get('{}/content?contentId={}'.format(
            self.domain,
            idx.group(1)
        ))

    def get_manga_name(self) -> str:
        return 'Fake'

    def prepare_cookies(self):
        self.__origin_url = self.get_url()

    def get_chapters(self):
        pass
        # return self._elements('.contlst-container .contlst-item > a')

    def get_files(self):
        return []

    def get_cover(self) -> str:
        pass

    def chapter_for_json(self):
        return self.get_url()


main = ComicoCoIdContent



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHereCc(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index().split('-'))

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        selector = r'/manga/[^/]+/[^\d]+(\d+)/[^\d]+(\d+)'
        idx = self.re.search(selector, chapter)
        if idx:
            return '-'.join(idx.groups())
        selector = r'/manga/[^/]+/[^\d]+(\d+)'
        return self.re.search(selector, chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.detail_list .left a')

    def get_files(self):
        selector = 'img#image'
        parser = self.html_fromstring(self.chapter)
        pages = self._first_select_options(parser, '.go_page select.wid60')
        images = self._images_helper(parser, selector)
        n = self.http().normalize_uri
        for page in pages:
            parser = self.html_fromstring(n(page.get('value')))
            images += self._images_helper(parser, selector)
        return images

    def get_cover(self):
        return self._cover_from_content('.manga_detail_top > img')

    def prepare_cookies(self):
        self._base_cookies()

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaHereCc



from manga_py.provider import Provider
from .helpers.std import Std


class ReadComicBooksOnlineOrg(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'/reader/[^/]+/[^/]+_(\d+(?:[\./]\d+)?)', self.chapter)
        if not idx:
            idx = self.re.search(r'/reader/[^/]+_(\d+(?:[\./]\d+)?)', self.chapter)
        return '-'.join(self.re.split(r'[/\.]', idx.group(1)))

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.(?:org|net)/(?:reader/)?([^/]+)')

    def get_chapters(self):
        return self._elements('#chapterlist .chapter > a')

    def prepare_cookies(self):
        self._storage['domain_uri'] = self.domain.replace('/www.', '/')

    def _get_image(self, parser):
        src = parser.cssselect()
        if not src:
            return None
        return '{}/reader/{}'.format(self.domain, src[0].get('src'))

    def get_files(self):
        parser = self.html_fromstring(self.chapter + '?q=fullchapter')
        n = self.http().normalize_uri
        print([n(i) for i in self._images_helper(parser, '#omv td > img')])
        print('')
        print(['{}/reader/{}'.format(self.domain, i) for i in self._images_helper(parser, '#omv td > img')])
        exit()
        return [self.chapter + i for i in self._images_helper(parser, '#omv td > img')]

    def get_cover(self):
        self._cover_from_content(self.content, '.pic > img.series')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = ReadComicBooksOnlineOrg



from manga_py.provider import Provider
from .helpers.std import Std


class JurnaluRu(Provider, Std):

    def get_archive_name(self) -> str:  # TODO ?
        # arc_name = self.re.search(
        #     '/{0}/{0}([^/]+)'.format(self.manga_name),
        #     self.chapter
        # )
        # if arc_name:
        #     pass
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return str(self._storage['current_chapter'])

    def get_main_content(self):
        name = self._get_name(r'(online-reading/[^/]+/[^/]+)')
        url = self.html_fromstring(
            '{}/{}'.format(self.domain, name),
            '.MagList .MagListLine > a',
            0
        ).get('href')
        return self.http_get(self.domain + url)

    def get_manga_name(self) -> str:
        return self._get_name(r'/online-reading/[^/]+/([^/]+)')

    def get_chapters(self):
        name = self.re.search(r'(online-reading/[^/]+/[^/]+)', self.get_url())
        if not name:
            return []
        items = self.document_fromstring(self.content, 'select.magSelection option')
        url = '{}/{}/'.format(self.domain, name.group(1))
        return [url + i.get('value') for i in items]

    @staticmethod
    def __get_file(parser):
        image = parser.cssselect('a[rel="shadowbox"]')
        return image[0].get('href')

    def get_files(self):
        chapter = self.chapter
        page = self.html_fromstring(chapter, '.ForRead', 0)
        pages = page.cssselect('.navigation')[0].cssselect('select.M option + option')
        images = [self.__get_file(page)]
        for i in pages:
            uri = '{}/{}'.format(chapter, i.get('value'))
            parser = self.html_fromstring(uri, '.ForRead', 0)
            images.append(self.__get_file(parser))
        return images

    def get_cover(self):
        return self._cover_from_content('.ops > div > img')


main = JurnaluRu



from manga_py.provider import Provider
from .helpers.std import Std


class VizCom(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.chapter_id)

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        url = self.re.search('/([^/]+/chapters/[^/]+)').group(1)
        return self.http_get('{}/{}'.format(self.domain, url))

    def get_manga_name(self) -> str:
        return self._get_name('/chapters/([^/]+)')

    def get_chapters(self):
        return self._elements('.o_products .chapter-text > a')

    def get_files(self):
        volume_id = self.re.search('/chapter/[^/]+/(\d+)', self.chapter)
        params = [
            'device%5Fid=3',
            # 'page={}',
            'manga%5Fid={}'.format(volume_id.groups()[0]),
            'loadermax=1',
        ]

        uri = '{}/manga/get_manga_url?'.format(self.domain)
        uri += '&'.join(params)

        n = 0
        _img_index = 0
        while n < 299:  # FIXME!

            _img_index += 1
            page_url = '{}&page={}'.format(uri, n)

            parser = self.html_fromstring(page_url, 'ImageLoader')

            if not len(parser):
                break

            # TODO!
            # t = MultiThreads()
            #
            # for i in parser:
            #     img_url = i.get('url')
            #     if ~img_url.find('blankpage.jpg'):
            #         break
            #     # see manga.py:280
            #     t.addThread(download_one_file, (img_url,))
            #     # safe_downloader(img_url, path.join(temp_root_path, 'img_{:0>3}.jpg'.format(_img_index)))
            # t.startAll()

            n += 2

        return [-1]

    def get_cover(self):
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = VizCom



from .authrone_com import AuthroneCom
from .helpers.std import Std


class ThreeAsqInfo(AuthroneCom, Std):
    _ch_selector = '.mng_det ul.lst > li > a.lst'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('.')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        return self.re.search(r'\.info/[^/]+/([^/]+)', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/{}/')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.info/([^/]+)')

    def get_files(self):
        return list(set(super().get_files()))  # remove doubles


main = ThreeAsqInfo



from urllib.parse import unquote_plus

from manga_py.provider import Provider
from .helpers.std import Std


class NightowNet(Provider, Std):
    _name_re = r'manga=(.+?)(?:&.+)?$'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index()
        return self.normal_arc_name(idx.split('-'))

    def get_chapter_index(self) -> str:
        ch = unquote_plus(self.chapter)
        idx = self.re.search(r'chapter=(?:.+?)\+(\d+(?:\.\d+)?)', ch)
        if idx:
            return '-'.join(idx.group(1).split('.'))
        return self.re.search('chapter=(.+?)(?:&.+)?$', ch).group(1)

    def get_main_content(self):
        name = self._get_name(self._name_re)
        return self.http_get('{}/online/?manga={}'.format(
            self.domain,
            name
        ))

    def get_manga_name(self) -> str:
        return unquote_plus(self._get_name(self._name_re))

    def get_chapters(self):
        return self._elements('.selector .options a')

    def prepare_cookies(self):
        self._storage['referer'] = self.domain + '/online/'

    def get_files(self):
        content = self.http_get(self.chapter)
        items = self.re.findall(r'imageArray\[\d+\]\s*=\s*[\'"](.+)[\'"];', content)
        n = self.http().normalize_uri
        return [n(i) for i in items]

    def get_cover(self) -> str:
        pass

    def book_meta(self) -> dict:
        # todo meta
        pass


main = NightowNet



import json
import sys
from argparse import ArgumentParser
from os import name as os_name
from getpass import getpass

import requests
from packaging import version
from progressbar import ProgressBar

from manga_py.meta import __version__, __repo_name__
from manga_py.parser import Parser
from manga_py.fs import check_free_space, get_temp_path


class Cli:  # pragma: no cover
    args = None
    parser = None
    _info = None
    __progress_bar = None

    def __init__(self, args: ArgumentParser, info=None):
        self.args = args.parse_args()
        self.parser = Parser(args)
        self._info = info

        space = self.args.min_free_space
        if not check_free_space(get_temp_path(), space) or not check_free_space(self.args.destination, space):
            raise OSError('No space left on device')

    def start(self):
        try:
            self.parser.init_provider(
                progress=self.progress,
                log=self.print,
                quest=self.quest,
                quest_password=self.quest_password,
                info=self._info,
            )
        except AttributeError as e:
            print(e)
            print('Please check the domain in the table: https://manga-dl.yuru-yuri.sttv.me')
            print('Make sure that the URL is correct\n')
            raise e
        self.parser.start()
        self.__progress_bar and self.__progress_bar.value > 0 and self.__progress_bar.finish()
        self.print(' ')

    def __init_progress(self, items_count: int, re_init: bool):
        if re_init or not self.__progress_bar:
            if re_init:
                self.__progress_bar.finish()
            bar = ProgressBar()
            self.__progress_bar = bar(range(items_count))
            self.__progress_bar.init()

    def progress(self, items_count: int, current_item: int, re_init: bool = False):
        if not items_count:
            return
        if not self.args.no_progress and not self.args.print_json:
            current_val = 0
            if self.__progress_bar:
                current_val = self.__progress_bar.value
            self.__init_progress(items_count, re_init and current_val > 0)
            self.__progress_bar.update(current_item)

    def print(self, text, **kwargs):
        if os_name == 'nt':
            text = str(text).encode().decode(sys.stdout.encoding, 'ignore')
        print(text, **kwargs)

    def _single_quest(self, variants, title):
        self.print(title)
        for v in variants:
            self.print(v)
        return input()

    def _multiple_quest(self, variants, title):
        self.print('Accept - blank line + enter')
        self.print(title)
        for v in variants:
            self.print(v)
        result = []
        while True:
            _ = input().strip()
            if not len(_):
                return result
            result.append(_)

    def quest(self, variants: enumerate, title: str, select_type=0):  # 0 = single, 1 = multiple
        if select_type:
            return self._multiple_quest(variants, title)
        return self._single_quest(variants, title)

    def quest_password(self, title):
        return getpass(title)



from .danbooru_donmai_us import DanbooruDonmaiUs
from .helpers.std import Std


class LoliBooruMoe(DanbooruDonmaiUs, Std):
    _archive_prefix = 'lolibooru_'

    def get_manga_name(self) -> str:
        if ~self.get_url().find(r'tags='):
            self._is_tag = True
            self._manga_name = self._get_name(r'[\?&]tags=([^&]+)')
        else:
            self._manga_name = self._get_name(r'/post/show/(\d+)')
        return self._archive_prefix + self._manga_name

    def get_chapters(self):  # pragma: no cover
        if self._is_tag:
            pages = self._elements('#paginator .pagination > a')
            if pages:
                count = self.re.search(r'\bpage=(\d+)', pages[-2].get('href')).group(1)
                max_page = int(count)
                if max_page > 1001:
                    self.log('1000 pages maximum!')
                    max_page = 1000
                return range(1, max_page)[::-1]
        return [1]

    def _tag_images(self):  # pragma: no cover
        url = '{}/post?tags={}&page={}'.format(
            self.domain,
            self._manga_name,
            self.chapter,
        )
        parser = self.html_fromstring(url)
        return self._images_helper(parser, '#post-list-posts a.directlink', 'href')

    def _post_image(self, url):  # pragma: no cover
        if isinstance(url, str):
            parser = self.html_fromstring(url)
        else:
            parser = url

        full_size = parser.cssselect('.status-notice a.highres-show')
        if full_size:
            return [full_size[0].get('href')]
        return [parser.cssselect('#image')[0].get('src')]

    def chapter_for_json(self):
        return self.get_url()


main = LoliBooruMoe



from time import sleep

from manga_py.fs import get_util_home_path, path_join, is_file
from .e_hentai_org import EHentaiOrg


class exhentai_org(EHentaiOrg):
    def prepare_cookies(self):
        super().prepare_cookies()

        cookie_file = path_join(get_util_home_path(), 'cookies_exhentai.dat')
        if is_file(cookie_file):
            with open(cookie_file, 'r') as r:
                self._storage['cookies'] = self.json.loads(r.read())
        else:
            # Login on e-hentai!
            name = self.quest([], 'Request login on e-hentai.org')
            password = self.quest_password('Request password on e-hentai.org')
            content = self.http_post('https://forums.e-hentai.org/index.php?act=Login&CODE=01', data={
                'CookieDate': 1,
                'b': 'd',
                'bt': '1-1',
                'ipb_login_submit': 'Login!',
                'UserName': name,
                'PassWord': password,
            })
            if not ~content.find('You are now logged in as:'):
                print('Wrong password?')
                sleep(.1)
                exit()
            else:
                with open(cookie_file, 'w') as w:
                    w.write(self.json.dumps(self._storage['cookies']))

            sleep(5)


main = exhentai_org



from .gomanga_co import GoMangaCo


class ReadPowerMangaOrg(GoMangaCo):
    _name_re = '[^/]/[^/]+/([^/]+)'
    _content_str = '{}/series/{}/'

    def get_chapter_index(self) -> str:
        url = self.chapter
        index_re = r'/rea\w+/[^/]+/(?:[^/]+/)?(\d+/\d+(?:/\d+)?)'
        group = self.re.search(index_re, url).group(1)
        return group.replace('/', '-')


main = ReadPowerMangaOrg



from manga_py.provider import Provider
from .helpers.std import Std


class MangaInnNet(Provider, Std):

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        chapter = self.chapter
        idx = self.re.search(r'\.net/[^/]+/([^/]+)', chapter).group(1).split('.')
        return '{}-{}'.format(*self._idx_to_x2(idx))

    def get_main_content(self):
        return self._get_content('{}/{}')

    def get_manga_name(self) -> str:
        name = self.re.search(r'\.net/([^/]+)', self.get_url())
        return name.group(1)

    def get_chapters(self):
        return self.document_fromstring(self.content, '#chapter_list a[href]')

    def get_files(self):
        content = self.http_get(self.chapter)
        images = self.re.search(r'var\s+images\s*=\s*(\[\{.+?\}\])', content).group(1)
        images = self.json.loads(images)
        return [i.get('url') for i in images]

    def get_cover(self):
        pass  # TODO

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaInnNet



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHasuSe(Provider, Std):

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index().split('-'))

    def get_chapter_index(self) -> str:
        re = self.re.compile('chapter-+(\d+)(?:-+(\d+))?')
        idx = re.search(self.chapter).groups()
        if idx[1] is not None:
            return '{}-{}'.format(*idx)
        return idx[0]

    def get_main_content(self):
        url = self.get_url()
        test = self.re.search(r'\.\w{2,5}/[^/]+-p\d+.html', url)
        if not test:
            self.cf_protect(self.get_url())
            url = self.html_fromstring(url, 'a.itemcrumb.active', 0).get('href')
        return self.http_get(url)

    def get_manga_name(self) -> str:
        return self._get_name(r'\.\w{2,5}/([^/]+)(?:-p\d+.html|/)')

    def get_chapters(self):
        return self._elements('.list-chapter .name a')

    def get_files(self):
        if not self._params.get('cf-protect'):
            self.cf_protect(self.chapter)
        parser = self.html_fromstring(self.chapter)
        return self._images_helper(parser, '.img > img')

    def get_cover(self) -> str:
        return self._cover_from_content('.info-img > img')

    def book_meta(self) -> dict:
        pass


main = MangaHasuSe



from .helpers.std import Std
from .mangaonline_today import MangaOnlineToday


class MangaDeepCom(MangaOnlineToday, Std):

    def get_chapter_index(self) -> str:
        idx = self.re.search(r'\.com/[^/]+/([^/]+)', self.chapter)
        return idx.group(1)

    def document_fromstring(self, body, selector: str = None, idx: int = None):
        if ~body.find('<?'):
            body = self.re.sub(r'<\?', '', body)
        return super().document_fromstring(body, selector, idx)

    def get_manga_name(self) -> str:
        return self._get_name(r'\.com/([^/]+)')

    def get_chapters(self):
        selector = 'ul.lst a.lst'
        items = self.document_fromstring(self.content, selector)
        pages = self.document_fromstring(self.content, '.pgg li > a')
        if pages:
            idx = self.re.search(r'-list/(\d+)', pages[-1].get('href'))
            for i in range(1, int(idx.group(1))):
                content = self.http_get('{}/{}/chapter-list/{}/'.format(
                    self.domain,
                    self.manga_name,
                    i + 1
                ))
                items += self.document_fromstring(content, selector)
        return items

    def get_cover(self) -> str:
        return self._cover_from_content('img.cvr')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaDeepCom



from manga_py.crypt import MangaRockComCrypt
from manga_py.fs import rename, unlink, basename
from manga_py.provider import Provider
from .helpers.std import Std


class MangaRockCom(Provider, Std):
    __name = None
    crypt = None
    __api_uri = 'https://api.mangarockhd.com/query/web400/'

    def get_archive_name(self) -> str:
        return self.normal_arc_name(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return str(self.chapter_id)

    def get_main_content(self):
        pass

    def get_manga_name(self) -> str:
        self.__name = self._get_name(r'/manga/([^/]+-\d+)')
        content = self.http_get('{}/manga/{}'.format(
            self.domain,
            self.__name
        ))
        return self.text_content(content, 'h1')

    def get_chapters(self):
        idx = self._get_name('/manga/([^/]+)')
        url = '{}info?oid={}&last=0&country=Japan'.format(self.__api_uri, idx)
        items = self.json.loads(self.http_get(url))
        return [(i.get('oid'),) for i in items.get('data', {}).get('chapters', [])][::-1]

    def __get_url(self):
        return '{}pages?oid={}&country=Japan'.format(self.__api_uri, self.chapter[0])

    def get_files(self):
        items = self.json.loads(self.http_get(self.__get_url()))
        return items.get('data')

    # decrypt
    def after_file_save(self, _path, idx: int):
        _path_wp = _path + 'wp'
        with open(_path, 'rb') as file_r:
            with open(_path_wp, 'wb') as file_w:
                file_w.write(self.crypt.decrypt(file_r.read()))
        unlink(_path)
        rename(_path_wp, _path)

    def save_file(self, idx=None, callback=None, url=None, in_arc_name=None):
        _path, idx, _url = self._save_file_params_helper(url, idx)
        in_arc_name = basename(_path) + '.webp'
        return super().save_file(idx, callback, _url, in_arc_name)

    def get_cover(self) -> str:
        selector = 'div:not([class]) > div[class] > div[class] > div[class] > div[class] > img'
        url = '{}{}'.format(self.domain, self._get_name('(/manga/[^/]+)'))
        img = self._elements(selector, self.http_get(url))
        if img and len(img):
            return img[0].get('src')

    def prepare_cookies(self):
        self.crypt = MangaRockComCrypt()

    def book_meta(self) -> dict:
        # todo meta
        pass

    def chapter_for_json(self):
        return self.get_url()


main = MangaRockCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaLifeUs(Provider, Std):
    img_selector = '.image-container .CurImage'

    def get_archive_name(self) -> str:
        idx = self.get_chapter_index().split('-')
        return self.normal_arc_name(idx)

    def get_chapter_index(self) -> str:
        selector = r'-chapter-(\d+).+-index-(\d+)'
        chapter = self.re.search(selector, self.chapter)
        if chapter is None:   # http://mangalife.us/manga/Ubau-Mono-Ubawareru-Mono  #51
            selector = r'-chapter-(\d+(?:\.\d+)?)'
            chapter = self.re.search(selector, self.chapter).group(1).split('.')
            return '-'.join(chapter)
        return '{}-{}'.format(
            1 if chapter[1] is None else chapter[1],  # todo: maybe 0 ?
            chapter[0]
        )

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        uri = self.get_url()
        test = uri.find('.us/read-online/') > 0
        if test:
            uri = self.html_fromstring(uri, 'a.list-link', 0).get('href')
        return self.re.search(r'(?:\.us)?/manga/([^/]+)', uri).group(1)

    def get_chapters(self):
        return self._elements('.chapter-list a.list-group-item')

    def get_files(self):
        url = self.chapter
        parser = self.html_fromstring(url, '.mainWrapper', 0)
        pages = parser.cssselect('select.PageSelect')[0].cssselect('option + option')
        images = self._images_helper(parser, self.img_selector)
        for page in pages:
            page_url = self.re.sub(r'(.+page-)\d+(.+)', r'\1{}\2', url)
            parser = self.html_fromstring(page_url.format(page.get('value')))
            images += self._images_helper(parser, self.img_selector)
        return images

    def get_cover(self) -> str:
        return self._cover_from_content('.leftImage img')

    def book_meta(self) -> dict:
        # todo meta
        pass


main = MangaLifeUs
